//: [Previous](@previous)

import Foundation

let someMarkdown = """
    # Hello
    This is markdown sample with
       [Some link](https://www.some-link.com.ua)
    Have *fun*!
"""

struct Link {
    var title: String
    var url: URL
}

extension Link: CustomStringConvertible {
    var description: String {
        return "Title: \(title), url: \(url)"
    }
}

extension Link {
    static func parse(markdown: String) -> [Link] {
        // Step 1: Create Regexp
        let regexp = try! NSRegularExpression(
            pattern: #"\[(.*)\]\((\S+)\)"#,
            options: []
        )

        // Step 2: Get matches
        let matches = regexp.matches(
            in: markdown, options: [],
            range: NSRange(location: 0, length: markdown.count)
        )

        // Step 3: Map each match to Link structure
        return matches.compactMap { match in
            return Link(markdown: markdown, match: match)
        }
    }
}

extension Link {
    init?(markdown: String, match: NSTextCheckingResult) {
        // Крок 1: отримуємо дані, як підрядки початковго тексту.
        guard
            match.numberOfRanges == 3,
            let titleString = markdown.substring(nsRange: match.range(at: 1)),
            let urlString   = markdown.substring(nsRange: match.range(at: 2))
        else {
            return nil
        }

        // Крок 2: валідуємо дані. Ми могли би описати валідацію URL в регулярках, але навіщо:
        // при створенні URL з рядка все одно відбувається валідація.
        guard let url = URL(string: String(urlString)) else { return nil }

        // Крок 3: Звичайна ініціалізація властивостей структури.
        self.title = String(titleString)
        self.url = url
    }
}

let links = Link.parse(markdown: someMarkdown)
print(links.map { $0.description }.joined(separator: "\n"))


//: [Next](@next)
